
#define F_CPU 16000000

#include <stdio.h>
#include <string.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>
#include <avr/wdt.h>
#include <avr/pgmspace.h>
#include <util/delay.h>

uint16_t frequency = 100;

uint16_t get_adc(uint8_t channel)
{
    uint16_t ADCL_Data, ADCH_Data, value;

    ADMUX &= ~0x07;
    ADMUX |= channel;

    ADCSRA |= (1 << ADSC);

    // Wait for conversion
    while (ADCSRA & (1 << ADSC))
    	continue;

    ADCL_Data = ADCL;
    ADCH_Data = ADCH;

    // Add to result
    value = ADCL_Data + (ADCH_Data << 8);

    return value;
}

void initADC()
{
    ADCSRA = (1 << ADEN);

    // Prescaler at 128
    ADCSRA |= (1 << ADPS0) | (1 << ADPS1) | (1 << ADPS2);

    ADMUX  = (1 << REFS0);
}

void initPWM()
{
    // Set Timer1 to 0
    TIMSK1 = 0;

    // Output on PD5
    DDRD |= (1 << PD5);

    TCCR1A |= (1 << COM1A1);

    // FastPWM
    TCCR1A |= (1 << WGM11);
    TCCR1B |= (1 << WGM13) | (1 << WGM12);

    // No prescaler
    TCCR1B |= (1 << CS10);

    OCR1A = 10;
    ICR1  = 0xF;

    TIMSK1 |= (1 << TOIE1);
}

/* Timer1 overflow routine */
ISR(TIMER1_OVF_vect)
{
     ICR1 = frequency;
}

void update(uint16_t adc)
{
    frequency = 25 + (adc >> 3);
}

int main(void)
{
    uint16_t value;

    TIMSK1 = 0;
	
    initADC();
    initPWM();

    sei();

    while(1)
	{
        value = get_adc(0);
        update(value);
    }

    return 0;
}

